
#include "../snippets.lua"



function init()
	snd = {
        begin = 0,
        middle = 0,
        stop = 0,
        rotate = 0
    }

    delay = {
        value = 0,
        default = 0.5
    }

    rotSpeed = 45
    rotSpeedRad = math.rad(rotSpeed)

    weapon = {
        force = 0.5,
        range = 100,
        type = "bullet"
    }

    fov = {}
    fov.y = 60 -- ver
    fov.x = math.deg(2 * math.atan(math.tan(fov.y / 2) * (16 / 9))) -- hor

    maxAngle = 45

    target = nil

    hinges = {
        x = FindJoint("x"),
        y = FindJoint("y"),
        limits = {
            x = {
                min = 0,
                max = 0
            },
            y = {
                min = 0,
                max = 0
            },
        }
    }
    hinges.limits.x.min, hinges.limits.x.max = -maxAngle, maxAngle--GetJointLimits(x)
    hinges.limits.y.min, hinges.limits.y.max = -maxAngle, maxAngle--GetJointLimits(y)


    shapes = {
        head = FindShape("head"),
        foot = FindShape("foot"),
        turret = FindShape("turret")
    }

    bodies = {
        head = GetShapeBody(shapes.head),
        foot = GetShapeBody(shapes.foot),
        turret = GetShapeBody(shapes.turret)
    }

    locations = {
        gun = FindLocation("gun")
    }


    STANDBYLEFT = 0
    AIM = 1
    RESET = 2
    STANDBYRIGHT = 3

    state = STANDBYLEFT

    tolerance = 5

    targetResetTime = {
        value = 0,
        default = 0.5
    }

    weapon = {
        range = 100,
        type = "bullet",
        strength = 0.5,
        cooldown = {
            value = 0,
            default = 0.1
        },
        spread = 0.015
    }

    friendly = false
    if GetTagValue(shapes.head, "friendly") == "true" then
        friendly = true
        weapon.type = "gun"
    end

    if GetTagValue(shapes.head, "type") == "rocket" then
        weapon.type = "rocket"
        weapon.strength = 1.5
        weapon.cooldown.default = 2.0
        weapon.spread = 0
    end

    salvo = {
        count = 0,
        max = 50,
        recover = false,
        cooldown = {
            value = 0,
            default = 2.0
        }
    }

    sndShoot = LoadSound("MOD/snd/shoot.ogg")
    
end

function tick(dt)
    if IsBodyBroken(bodies.turret) then
        SetLightEnabled(FindLight(""), false)
        return
    end
    standby()
	udpateState(dt)
    targetResetTime.value = targetResetTime.value - dt
    if targetResetTime.value <= 0 then
        updateTarget()
    end
    doAction(dt)
    drawHeadDir()
end

function update(dt)
	--udpateState(dt)
end

function draw()
	
end

function static()
    SetJointMotorTarget(hinge, 0, 3)
end

function rotate()
    SetJointMotorTarget(hinge, 0, rotSpeedRad)
end

function udpateState(dt)
    if target ~= nil then
        if not IsHandleValid(target) and target ~= "player" then
            target = nil
            udpateState(dt)
        else
            state = AIM
        end

    elseif state == STANDBYLEFT then
        local angle = GetJointMovement(hinges.y)
        if angle <= hinges.limits.y.min + tolerance then
            state = STANDBYRIGHT
        end

    elseif state == STANDBYRIGHT then
        local angle = GetJointMovement(hinges.y)
        if angle >= hinges.limits.y.max - tolerance then
            state = STANDBYLEFT
        end
    
    elseif state == AIM then
        if target == nil then
            state = STANDBYLEFT
        end
    end
end

function doAction(dt)
    --DebugPrint(state == AIM)
    if target == nil and salvo.count > 0 then
        salvo.count = salvo.count - 1
    end
    if salvo.recover then
        salvo.cooldown.value = salvo.cooldown.value - dt
        if salvo.cooldown.value <= 0 then
            salvo.recover = false
            salvo.count = 0
        end
        return
    end
    if state == STANDBYLEFT or state == STANDBYRIGHT then
        standby()
    elseif state == AIM then
        aim()
        shoot(dt)
    end
end

function standby()
    if state == STANDBYLEFT then
        SetJointMotorTarget(hinges.y, hinges.limits.y.min, rotSpeedRad)
    elseif state == STANDBYRIGHT then
        SetJointMotorTarget(hinges.y, hinges.limits.y.max, rotSpeedRad)
    end
    SetJointMotorTarget(hinges.x, 0, rotSpeedRad)
end

function shoot(dt)

    weapon.cooldown.value = weapon.cooldown.value - dt

    local ht = GetBodyTransform(bodies.foot)--GetBodyTransform(bodies.head)
    ht.pos = GetBodyTransform(bodies.head).pos
    local headDir = TransformToParentVec(ht, fwd())
    local headQuat = QuatLookAt(Vec(), headDir)
    local toTarget = Vec()

    if target == "player" then
        vel = GetPlayerVelocity()
        local pos = TransformToLocalPoint(ht, GetPlayerCameraTransform().pos)
        lookAt = QuatLookAt(Vec(), pos)
        toTarget = pos
    else
        vel = GetBodyVelocity(target)
        local pos = TransformToLocalPoint(ht, GetBodyTransform(target).pos)
        lookAt = QuatLookAt(Vec(), pos)
        toTarget = pos
    end
    toTarget = VecScale(toTarget, -1)
    local tt = GetBodyTransform(bodies.head)
    local trueHeadDir = TransformToParentVec(tt, fwd())

    if weapon.cooldown.value <= 0 and vecAngle(headDir, toTarget) * 0 <= 10 then
        weapon.cooldown.value = weapon.cooldown.default
        local spreadVec = randVec(weapon.spread)
        spreadVec[3] = 0
        spreadVec = TransformToParentVec(tt, spreadVec)
        salvo.count = salvo.count + 1
        if salvo.count >= salvo.max then
            salvo.recover = true
            salvo.cooldown.value = salvo.cooldown.default
        end
        PlaySound(sndShoot, tt.pos, 0.5)
        Shoot(TransformToParentPoint(tt, Vec(0, 0.3, -1.2)), VecNormalize(VecAdd(trueHeadDir, spreadVec)), weapon.type, weapon.strength, weapon.range)
    end
end

function aim()
    if target ~= nil then
        local vel, toTarget
        local lookAt

        local ht = GetBodyTransform(bodies.foot)--GetBodyTransform(bodies.head)
        ht.pos = GetBodyTransform(bodies.head).pos
        local headDir = TransformToParentVec(ht, fwd())
        local headQuat = QuatLookAt(Vec(), headDir)
        local toTarget = Vec()

        if target == "player" then
            vel = GetPlayerVelocity()
            local pos = TransformToLocalPoint(ht, GetPlayerCameraTransform().pos)
            lookAt = QuatLookAt(Vec(), pos)
            toTarget = pos
        else
            vel = GetBodyVelocity(target)
            local pos = TransformToLocalPoint(ht, GetBodyTransform(target).pos)
            lookAt = QuatLookAt(Vec(), pos)
            toTarget = pos
        end

        local rx, ry, rz = GetQuatEuler(lookAt)--getFloorQuat(QuatRotateQuat(lookAt, QuatEuler(0, 180, 0)))
        ry = -ry
        rx = rx
        --DebugWatch("rot", rx .. " " .. ry.. " " .. rz)--quatToStr(floorQuat(lookAt)))

        SetJointMotorTarget(hinges.y, ry, rotSpeedRad)
        SetJointMotorTarget(hinges.x, rx, rotSpeedRad)
    end
end

function drawHeadDir()
    local ht = GetBodyTransform(bodies.head)
    --DrawLine(ht.pos, TransformToParentPoint(ht, Vec(0, 0, -3)))
    if target ~= nil then
        if target == "player" then
            --DrawLine(ht.pos, GetPlayerCameraTransform().pos, 1)
        else
            --DrawLine(ht.pos, GetBodyTransform(target).pos, 1)
        end
    end
end

function isSelf(element)
    return element == bodies.head or element == bodies.turret or element == bodies.foot
end

function updateTarget()

    local ht = GetBodyTransform(bodies.foot)--GetBodyTransform(bodies.head)
    ht.pos = GetBodyTransform(bodies.head).pos
    local headDir = TransformToParentVec(ht, fwd())

    if target ~= nil then
        local vel, toTarget
        if target == "player" then
            vel = GetPlayerVelocity()
            toTarget = VecSub(GetPlayerCameraTransform().pos, ht.pos)
        else
            vel = GetBodyVelocity(target)
            toTarget = VecSub(GetBodyTransform(target).pos, ht.pos)
        end

        if not targetValid(headDir, toTarget, vel) then
            target = nil
        end
    end

    if target == nil then
        local bmu = {
            handle = nil,
            value = 0
        }
        local bodies = FindBodies("", true)
        bodies[#bodies + 1] = "player"
        for i=1, #bodies do
            local body = bodies[i]
            local vel, toTarget
            if body == "player" then
                vel = GetPlayerVelocity()
                toTarget = VecSub(GetPlayerCameraTransform().pos, ht.pos)
            else
                vel = GetBodyVelocity(body)
                toTarget = VecSub(GetBodyTransform(body).pos, ht.pos)
            end
            if (body == "player" or IsBodyDynamic(body)) and targetValid(headDir, toTarget, vel, ht.pos, body) then
                local angle = vecAngle(toTarget, headDir)
                if bmu.handle == nil or angle < bmu.value then
                    bmu.handle = body
                    bmu.value = angle
                end
            end
        end
        target = bmu.handle
    end
    --target = "player"
    if target ~= nil then
        targetResetTime.value = targetResetTime.default
    end
    --DrawBodyOutline(target, 1, 1, 1, 1)
end

function targetValid(dir, toTarget, vel, origin, body)
    if friendly and body == "player" then
        return false
    end
    if (VecLength(vel) > 3 or body == "player") and vecAngle(toTarget, dir) <= maxAngle and not isSelf(body) and VecLength(toTarget) <= 40 then
        QueryRejectBody(body)
        QueryRejectBody(bodies.head)
        QueryRejectBody(bodies.turret)
        local hit = abRaycast(origin, VecAdd(origin, toTarget), false, 0, true)
        return not hit
    end
    return false
end